@@ -1,5 +1,5 @@
 #
-#   xts: eXtensible time-series 
+#   xts: eXtensible time-series
 #
 #   Copyright (C) 2008  Jeffrey A. Ryan jeff.a.ryan @ gmail.com
 #
@@ -19,15 +19,19 @@
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+#' @rdname first
 `last` <-
 function(x,...)
 {
   UseMethod("last")
 }
 
+#' @rdname first
 `last.default` <-
 function(x,n=1,keep=FALSE,...)
 {
+  if(length(x) == 0)
+    return(x)
   if(is.character(n)) {
     xx <- try.xts(x, error=FALSE)
     if(is.xts(xx)) {
@@ -37,27 +41,40 @@ function(x,n=1,keep=FALSE,...)
   }
   if(is.null(dim(x))) {
     if(n > 0) {
-      xx <- x[(NROW(x)-n+1):NROW(x)]
+      sub <- seq.int(to = length(x), length.out = min(n, length(x)))
+      xx <- x[sub]
       if(keep) xx <- structure(xx,keep=x[1:(NROW(x)+(-n))])
       xx
-    } else {
-      xx <- x[1:(NROW(x)+n)]
+    } else if(n < 0) {
+      sub <- seq_len(max(length(x) + n, 0L))
+      xx <- x[sub]
       if(keep) xx <- structure(xx,keep=x[((NROW(x)-(-n)+1):NROW(x))])
       xx
+    } else {
+      xx <- x[0]
+      if(keep) xx <- structure(xx,keep=x[0])
+      xx
     }
   } else {
     if(n > 0) {
-      xx <- x[(NROW(x)-n+1):NROW(x),,drop=FALSE]
+      sub <- seq.int(to = NROW(x), length.out = min(n, NROW(x)))
+      xx <- x[sub,,drop=FALSE]
       if(keep) xx <- structure(xx,keep=x[1:(NROW(x)+(-n)),])
       xx
-    } else {
-      xx <- x[1:(NROW(x)+n),,drop=FALSE]
+    } else if(n < 0) {
+      sub <- seq_len(max(NROW(x) + n, 0L))
+      xx <- x[sub,,drop=FALSE]
       if(keep) xx <- structure(xx,keep=x[((NROW(x)-(-n)+1):NROW(x)),])
       xx
+    } else {
+      xx <- x[0,,drop=FALSE]
+      if(keep) xx <- structure(xx,keep=x[0,])
+      xx
     }
   }
 }
 
+#' @rdname first
 `last.xts` <-
 function(x,n=1,keep=FALSE,...)
 {
@@ -70,10 +87,11 @@ function(x,n=1,keep=FALSE,...)
       stop(paste("incorrectly specified",sQuote("n"),sep=" "))
     # series periodicity
     sp <- periodicity(x)
+    sp.units <- sp[["units"]]
     # requested periodicity$units
     rpu <- np[length(np)]
     rpf <- ifelse(length(np) > 1, as.numeric(np[1]), 1)
-    if(rpu == sp$unit) {
+    if(rpu == sp.units) {
       n <- rpf
     } else {
       # if singular - add an s to make it work
@@ -86,7 +104,7 @@ function(x,n=1,keep=FALSE,...)
       if(!rpu %in% dt.options)
         stop(paste("n must be numeric or use",paste(dt.options,collapse=',')))
       dt <- dt.options[pmatch(rpu,dt.options)]
-      if(u.list[[dt]] > u.list[[sp$unit]]) {
+      if(u.list[[dt]] > u.list[[sp.units]]) {
         #  req is for higher freq data period e.g. 100 mins of daily data
         stop(paste("At present, without some sort of magic, it isn't possible",
              "to resolve",rpu,"from",sp$scale,"data"))
@@ -98,28 +116,60 @@ function(x,n=1,keep=FALSE,...)
       }
       if(rpf > 0) {
         n <- ep[length(ep)-rpf]+1
-        xx <- x[n:NROW(x),,drop=FALSE]
+        if(is.null(dim(x))) {
+          xx <- x[n:NROW(x)]
+        } else {
+          xx <- x[n:NROW(x),,drop=FALSE]
+        }
         if(keep) xx <- structure(xx,keep=x[1:(ep[length(ep)+(-rpf)])])
         return(xx)
-      } else {
+      } else if(rpf < 0) {
         n <- ep[length(ep)+rpf]
-        xx <- x[1:n,,drop=FALSE]
+        if(is.null(dim(x))) {
+          xx <- x[1:n]
+        } else {
+          xx <- x[1:n,,drop=FALSE]
+        }
         if(keep) xx <- structure(xx,keep=x[(ep[length(ep)-(-rpf)]+1):NROW(x)])
         return(xx)
+      } else {
+        if(is.null(dim(x))) {
+          xx <- x[0]
+        } else {
+          xx <- x[0,,drop=FALSE]
+        }
+        if(keep) xx <- structure(xx,keep=x[0])
+        return(xx)
       }
     }
   }
   if(length(n) != 1) stop("n must be of length 1")
   if(n > 0) {
     n <- min(n, NROW(x))
-    xx <- x[(NROW(x)-n+1):NROW(x),,drop=FALSE]
+    if(is.null(dim(x))) {
+      xx <- x[(NROW(x)-n+1):NROW(x)]
+    } else {
+      xx <- x[(NROW(x)-n+1):NROW(x),,drop=FALSE]
+    }
     if(keep) xx <- structure(xx,keep=x[1:(NROW(x)+(-n))])
     xx
-  } else {
+  } else if(n < 0) {
     if(abs(n) >= NROW(x))
       return(x[0])
-    xx <- x[1:(NROW(x)+n),,drop=FALSE]
+    if(is.null(dim(x))) {
+      xx <- x[1:(NROW(x)+n)]
+    } else {
+      xx <- x[1:(NROW(x)+n),,drop=FALSE]
+    }
     if(keep) xx <- structure(xx,keep=x[((NROW(x)-(-n)+1):NROW(x))])
     xx
+  } else {
+    if(is.null(dim(x))) {
+      xx <- x[0]
+    } else {
+      xx <- x[0,,drop=FALSE]
+    }
+    if(keep) xx <- structure(xx,keep=x[0])
+    xx
   }
 }
