@@ -1,3 +1,25 @@
+/*
+#   xts: eXtensible time-series
+#
+#   Copyright (C) 2008  Jeffrey A. Ryan jeff.a.ryan @ gmail.com
+#
+#   Contributions from Joshua M. Ulrich
+#
+#   This program is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 2 of the License, or
+#   (at your option) any later version.
+#
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+
 #include <R.h>
 #include <Rinternals.h>
 #include "xts.h"
@@ -19,6 +41,14 @@ SEXP extract_col (SEXP x, SEXP j, SEXP drop, SEXP first_, SEXP last_) {
   SEXP result, index, new_index;
   int nrs, nrsx, i, ii, jj, first, last;
 
+  /* ensure 'j' is integer with length > 0 */
+  if(length(j) == 0) {
+    error("'j' cannot have zero length");
+  }
+  if(asInteger(j) == 0) {
+    error("'j' cannot equal 0");
+  }
+
   nrsx = nrows(x);
 
   first = asInteger(first_)-1;
@@ -119,7 +149,7 @@ Rprintf("i=%i, j=%i, nrs=%i, first=%i\n", i, INTEGER(j)[i]-1, nrs, first);
   if(nrs != nrows(x)) {
     copyAttributes(x, result);
     /* subset index */
-    index = getAttrib(x, xts_IndexSymbol);
+    index = PROTECT(getAttrib(x, xts_IndexSymbol));
     PROTECT(new_index = allocVector(TYPEOF(index), nrs)); 
     if(TYPEOF(index) == REALSXP) {
       memcpy(REAL(new_index), &(REAL(index)[first]), nrs*sizeof(double)); 
@@ -128,7 +158,7 @@ Rprintf("i=%i, j=%i, nrs=%i, first=%i\n", i, INTEGER(j)[i]-1, nrs, first);
     }
     copyMostAttrib(index, new_index);
     setAttrib(result, xts_IndexSymbol, new_index);
-    UNPROTECT(1);
+    UNPROTECT(2);
   } else {
     copyMostAttrib(x, result); /* need an xts/zoo equal that skips 'index' */
   }
