@@ -57,10 +57,10 @@ SEXP do_rbind_xts (SEXP x, SEXP y, SEXP dup)
     return y;
   }
 
-  if( !isXts(x) ) {
+  if( !Rf_asInteger(isXts(x)) ) {
     PROTECT( x = tryXts(x) ); P++;
   }
-  if( !isXts(y) ) {
+  if( !Rf_asInteger(isXts(y)) ) {
     PROTECT( y = tryXts(y) ); P++;
   }
 
@@ -102,11 +102,12 @@ if(TYPEOF(xindex)==INTSXP) {
 }
 #endif
 
+  if(nrx != length(xindex) || nry != length(yindex))
+    error("zero-length vectors with non-zero-length index are not allowed");
+
   PROTECT(newindex = allocVector(TYPEOF(xindex), len)); P++;
   PROTECT(result   = allocVector(TYPEOF(x), len * ncx)); P++;
 
-  copyMostAttrib(xindex, newindex);
-
   switch( TYPEOF(x) ) {
     case INTSXP:
         int_x = INTEGER(x);
@@ -487,7 +488,7 @@ return(result);
   SEXP dim;
   PROTECT(dim = allocVector(INTSXP, 2));
   INTEGER(dim)[0] = truelen;
-  INTEGER(dim)[1] = INTEGER(getAttrib(x, R_DimSymbol))[1];
+  INTEGER(dim)[1] = ncx;
   UNPROTECT(1);
   setAttrib(result, R_DimSymbol, dim);
   setAttrib(result, R_DimNamesSymbol, getAttrib(x, R_DimNamesSymbol));
@@ -495,10 +496,9 @@ return(result);
   if(truelen != len) {
     PROTECT(newindex = lengthgets(newindex, truelen)); P++;
   }
+  copyMostAttrib(xindex, newindex);
+
   setAttrib(result, xts_IndexSymbol, newindex);
-  setAttrib(result, xts_IndexClassSymbol, getAttrib(x, xts_IndexClassSymbol));
-  setAttrib(result, xts_IndexTZSymbol, getAttrib(x, xts_IndexTZSymbol));
-  setAttrib(result, xts_IndexFormatSymbol, getAttrib(x, xts_IndexFormatSymbol));
   setAttrib(result, xts_ClassSymbol, getAttrib(x, xts_ClassSymbol));
   copy_xtsAttributes(x, result);
   UNPROTECT(P);
@@ -625,8 +625,8 @@ SEXP rbind_append (SEXP x, SEXP y) {
   copyAttributes(x, result); 
 
   SEXP index, xindex, yindex;
-  xindex = getAttrib(x, xts_IndexSymbol);
-  yindex = getAttrib(y, xts_IndexSymbol);
+  xindex = PROTECT(getAttrib(x, xts_IndexSymbol));
+  yindex = PROTECT(getAttrib(y, xts_IndexSymbol));
   int INDEXTYPE = TYPEOF(xindex);
   if(INDEXTYPE != NILSXP) {
     PROTECT(index = allocVector(INDEXTYPE, nr));
@@ -668,6 +668,6 @@ SEXP rbind_append (SEXP x, SEXP y) {
     UNPROTECT(2);
 */
 
-  UNPROTECT(1);
+  UNPROTECT(3);
   return result;
 }
